use std::{process::exit, path::PathBuf, fs::File, io::{Read, Write}};

fn main() {
    let from = std::env::args().nth(1).unwrap_or_else(|| {
        println!("You gotta give me a binary file to slap the hex out of");
        exit(1);
    });

    let into = std::env::args().nth(2).unwrap_or_else(|| {
        println!("I need a file to put these values into");
        exit(1);
    });

    let output = std::env::args().nth(3).unwrap_or_else(|| {
        println!("I need a file to output to");
        exit(1);
    });

    let from_path: PathBuf = (&from).into();
    let into_path: PathBuf = (&into).into();
    let output_path: PathBuf = (&output).into();

    let mut from_file = File::open(&from_path).unwrap_or_else(|e| {
        println!("Error opening {from_path:?}: {e}");
        exit(1);
    });

    let mut into_file = File::open(&into_path).unwrap_or_else(|e| {
        println!("Error opening {into_path:?}: {e}");
        exit(1);
    });

    let mut output_file = File::create(&output_path).unwrap_or_else(|e| {
        println!("Error creating {output_path:?}: {e}");
        exit(1);
    });

    let mut from_contents: Vec<u8> = Vec::with_capacity(128);
    let mut into_contents: String = String::with_capacity(1024);    

    from_file.read_to_end(&mut from_contents).unwrap_or_else(|e| {
        println!("Failed to read {from_path:?} {e}");
        exit(1);
    });

    into_file.read_to_string(&mut into_contents).unwrap_or_else(|e| {
        println!("Failed to read {into_path:?}: {e}");
        exit(1);
    });

    let mem_value: String = from_contents.iter().map(|byte| {
        format!("{byte:02X}")
    }).collect();

    let contents_len = from_contents.len() * 8;
    let bit_len = (2048 * 8).max(contents_len * 2);
    let padding_len = bit_len - contents_len;

    let insert_mem = format!("reg [0:{}] mem = {{{contents_len}'h{mem_value}, {padding_len}'h0}};", bit_len - 1);

    into_contents = into_contents.replace("$$insert_mem$$", &insert_mem);

    output_file.write_all(into_contents.as_bytes()).unwrap_or_else(|e| {
        println!("Failed writing to {output_path:?}: {e}");
        exit(1);
    });
}
